<?php declare(strict_types=1);

namespace KorElf\TranslateLaravel\Translate;

use Illuminate\Contracts\Foundation\Application;
use Illuminate\Foundation\Bus\PendingDispatch;
use KorElf\TranslateLaravel\Contracts\CallbackAfterTranslated;
use KorElf\TranslateLaravel\Contracts\Translate;
use InvalidArgumentException;
use KorElf\TranslateLaravel\Contracts\TranslationCompletedListener;
use KorElf\TranslateLaravel\DTO\RunTranslateDto;
use KorElf\TranslateLaravel\Jobs\RunTranslate;

final class TranslateManager
{
    /**
     * The array of resolved translation services.
     */
    private array $translates = [];

    public function __construct(
        private Application $app,
    ) { }

    public function service(?string $name = null): Translate
    {
        $name = $name ?: $this->getDefaultDriver();

        return $this->translates[$name] ??= $this->resolve($name);
    }

    public function getDefaultDriver(): string
    {
        return $this->app['config']['translate.default'];
    }

    public function setDefaultDriver(string $name): void
    {
        $this->app['config']['translate.default'] = $name;
    }

    /**
     * Resolve the given translate service.
     *
     * @throws InvalidArgumentException
     */
    public function resolve(string $name): Translate
    {
        $config = $this->getConfig($name);

        if (\is_null($config)) {
            throw new InvalidArgumentException('Translate service [' . $name . '] is not defined.');
        }

        return $config['driver']::init($this->app, $config['config']);
    }

    /**
     * Disconnect the given driver and remove from local cache.
     */
    public function purge(?string $name = null): void
    {
        $name ??= $this->getDefaultDriver();

        unset($this->translates[$name]);
    }

    /**
     * Run through queues.
     */
    public function runJob(RunTranslateDto $params, TranslationCompletedListener $completedListener): PendingDispatch
    {
        return RunTranslate::dispatch($params, $completedListener);
    }

    public function getLimit(?string $driver = null): array
    {
        if ($driver === null) {
            $driver = $this->getDefaultDriver();
        }

        $config = $this->getConfig($driver);

        return $config['config']['limit'] ?? [];
    }

    /**
     * Dynamically call the default driver instance.
     */
    public function __call(string $method, mixed $parameters): mixed
    {
        return $this->service()->$method(...$parameters);
    }

    private function getConfig(string $name): ?array
    {
        return $this->app['config']["translate.services.{$name}"] ?? null;
    }
}